--------------------------------------------------------------------------------
--
-- Quick'n'dirty test bench
--
--------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
ENTITY TB_IDE_EXI IS
END TB_IDE_EXI;
 
ARCHITECTURE behavior OF TB_IDE_EXI IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
    COMPONENT IDE_EXI
    PORT(
         ide_adr : OUT  std_logic_vector(4 downto 0);
         ide_dat : INOUT  std_logic_vector(15 downto 0);
         ide_rd : OUT  std_logic;
         ide_wr : OUT  std_logic;
         exi_in : IN  std_logic;
         exi_out : OUT  std_logic;
         exi_clk : IN  std_logic;
         exi_cs : IN  std_logic
        );
    END COMPONENT;
   

   --Inputs
   signal exi_in : std_logic := '0';
   signal exi_clk : std_logic := '0';
   signal exi_cs : std_logic := '1';

	--BiDirs
   signal ide_dat : std_logic_vector(15 downto 0);

 	--Outputs
   signal ide_adr : std_logic_vector(4 downto 0);
   signal ide_rd : std_logic;
   signal ide_wr : std_logic;
   signal exi_out : std_logic;

   -- Clock period definitions
   constant exi_clk_period : time := 10 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: IDE_EXI PORT MAP (
          ide_adr => ide_adr,
          ide_dat => ide_dat,
          ide_rd => ide_rd,
          ide_wr => ide_wr,
          exi_in => exi_in,
          exi_out => exi_out,
          exi_clk => exi_clk,
          exi_cs => exi_cs
        );

   -- Clock process definitions
   exi_clk_process :process
   begin
		exi_clk <= '1';
		wait for exi_clk_period/2;
		exi_clk <= '0';
		wait for exi_clk_period/2;
   end process;
 

   -- Stimulus process
   stim_proc: process
   begin		
      -- insert stimulus here
		
		exi_cs <= '1';
		wait for exi_clk_period;

		-- exi_get_id()
		exi_in <= '0'; -- read
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '0'; -- 8 bit
		wait for exi_clk_period;
		exi_in <= '0'; -- multi
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		wait for 31*exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

		-- read 8-bits
		exi_in <= '0'; -- read
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '0'; -- 8 bit
		wait for exi_clk_period;
		exi_in <= '0'; -- multi
		wait for exi_clk_period;
		exi_in <= '1'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		wait for exi_clk_period;
		ide_dat <= "0000000100000001";
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		wait for 7*exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

		-- read 16-bits
		exi_in <= '0'; -- read
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '1'; -- 16 bit
		wait for exi_clk_period;
		exi_in <= '0'; -- multi
		wait for exi_clk_period;
		exi_in <= '1'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		wait for exi_clk_period;
		ide_dat <= "0000001100000011";
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		wait for 15*exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

		-- write 8-bits
		exi_in <= '1'; -- write
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '0'; -- 8 bit
		wait for exi_clk_period;
		exi_in <= '0'; -- multi
		wait for exi_clk_period;
		exi_in <= '1'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

		-- write 16-bits
		exi_in <= '1'; -- write
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '1'; -- 16 bit
		wait for exi_clk_period;
		exi_in <= '0'; -- multi
		wait for exi_clk_period;
		exi_in <= '1'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[14]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[13]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[12]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[11]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[10]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[9]
		wait for exi_clk_period;
		exi_in <= '0'; -- data[8]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

		-- multi-read, 3x 32-bit words
		exi_in <= '0'; -- read
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '0'; -- 8 bit
		wait for exi_clk_period;
		exi_in <= '1'; -- multi
		wait for exi_clk_period;
		exi_in <= '1'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[4]
--		ide_dat <= "0000000000000000";
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- count_LSB[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- count_LSB[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		ide_dat <= "0000000100000001";
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		wait for 11*exi_clk_period;
		ide_dat <= "0000001100000011";
		wait for 16*exi_clk_period;
		ide_dat <= "0000011100000111";
		wait for 5*exi_clk_period;

		wait for 11*exi_clk_period;
		ide_dat <= "0000111100001111";
		wait for 16*exi_clk_period;
		ide_dat <= "0001111100011111";
		wait for 5*exi_clk_period;

		wait for 11*exi_clk_period;
		ide_dat <= "1111111111111111";
		wait for 16*exi_clk_period;
		ide_dat <= "1111111111111111";
		wait for 5*exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

		-- multi-write, 3x 32-bit words
		exi_in <= '1'; -- read
		exi_cs <= '0';
		wait for exi_clk_period;
		exi_in <= '0'; -- 8 bit
		wait for exi_clk_period;
		exi_in <= '1'; -- multi
		wait for exi_clk_period;
		exi_in <= '1'; -- reg[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- reg[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[4]
--		ide_dat <= "0000000000000000";
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_LSB[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- count_LSB[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- count_LSB[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- count_MSB[0]
		wait for exi_clk_period;

		exi_in <= '0'; -- data1[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[14]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[13]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[12]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[11]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[10]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[9]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[8]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[14]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[13]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[12]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[11]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[10]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[9]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[8]
		wait for exi_clk_period;
		
		exi_in <= '0'; -- data1[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[3]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[14]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[13]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[12]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[11]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[10]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[9]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[8]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[4]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[3]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[14]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[13]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[12]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[11]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[10]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[9]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[8]
		wait for exi_clk_period;

		exi_in <= '0'; -- data1[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[5]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[4]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[3]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[14]
		wait for exi_clk_period;
		exi_in <= '0'; -- data1[13]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[12]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[11]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[10]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[9]
		wait for exi_clk_period;
		exi_in <= '1'; -- data1[8]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[6]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[5]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[4]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[3]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[2]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[1]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[0]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[15]
		wait for exi_clk_period;
		exi_in <= '0'; -- data0[14]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[13]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[12]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[11]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[10]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[9]
		wait for exi_clk_period;
		exi_in <= '1'; -- data0[8]
		wait for exi_clk_period;

		exi_in <= '0'; -- zero[7]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[6]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[5]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[4]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[3]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[2]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[1]
		wait for exi_clk_period;
		exi_in <= '0'; -- zero[0]
		wait for exi_clk_period;

		exi_cs <= '1';
		wait for 10*exi_clk_period;

      wait;
   end process;

END;
